# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that updates the striim playlist database
when metadata information in the stations database changes.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Updater(object):
	"""
	This class updates the playlist database when metadata in the
	stations database changes.
	"""

	def __init__(self, manager):
		"""
		Initialize object.

		@param self: Reference to the Updater instance.
		@type self: An Updater object.
		"""
		self.__init_attributes(manager)
		self.__signal_id_1 = manager.connect("destroy", self.__destroy_cb)
		# Monitor database for changes.
		from gnomevfs import monitor_add, MONITOR_FILE
		self.__monitor_id = monitor_add(self.__database_uri, MONITOR_FILE, self.__database_changed_cb)


	def __init_attributes(self, manager):
		"""
		Initialize data attributes.

		@param self: Reference to the Updater instance.
		@type self: An Updater object.
		"""
		self.__manager = manager
		# Path to the stations database.
		from Striim.Global import metadata_folder
		self.__database_path = metadata_folder + "/stations.gdb"
		from gnomevfs import get_uri_from_local_path
		self.__database_uri = get_uri_from_local_path(self.__database_path)
		self.__monitor_id = self.__signal_id_1 = None
		return

	def __update_playlist_database(self):
		"""
		Update the playlist database.

		@param self: Reference to the Updater instance.
		@type self: An Updater object.
		"""
		from Metadata import get_stations_from_database
		stations = get_stations_from_database()
		from operator import not_, eq
		if not_(stations): return
		from Metadata import get_playlist_from_database
		playlist = get_playlist_from_database()
		if not_(playlist): return
		station_name = playlist.keys()[0]
		try:
			playlist_metadata = playlist[station_name]
			station_metadata = stations[station_name]
		except KeyError:
			return
		if eq(playlist_metadata, station_metadata): return
		playlist.clear()
		playlist[station_name] = station_metadata
		from Metadata import update_playlist_database
		update_playlist_database(playlist)
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Updater instance.
		@type self: An Updater object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__manager)
		from gnomevfs import monitor_cancel
		if self.__monitor_id: monitor_cancel(self.__monitor_id)
		delete_attributes(self)
		del self
		self = None
		return

	def __database_changed_cb(self, monitor_uri, info_uri, event_type):
		"""
		Handles callback when the abbreviation database is modified.

		@param self: Reference to the TemplateLoader instance.
		@type self: An TemplateLoader object.

		@param monitor_uri: The uri that is monitored.
		@type monitor_uri: A String object.

		@param info_uri: The uri that is monitored.
		@type info_uri: A String object.

		@param event_type: The type of modification that occured.
		@type event_type: A gnomevfs.MONITOR_EVENT* object.
		"""
		from gnomevfs import MONITOR_EVENT_CHANGED
		from operator import contains
		events = [MONITOR_EVENT_CHANGED]
		if contains(events, event_type):
			self.__update_playlist_database()
		return
