# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that monitor the mouse pointers movement
and emits a signal when it's on the player's status icon.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Cursor(object):
	"""
	This class monitor's the mouse pointer's movement and emits a signal
	when it's on the player's status icon.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the Cursor instance.
		@type self: A Cursor object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(manager, player)
		self.__precompile_methods()
		from gobject import timeout_add, PRIORITY_LOW
		timeout_add(500, self.__monitor_cursor_movement, priority=PRIORITY_LOW)
		self.__signal_id_1 = manager.connect("icon-position", self.__icon_position_cb)
		self.__signal_id_2 = manager.connect("destroy", self.__destroy_cb)

	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Cursor instance.
		@type self: A Cursor object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__manager = manager
		self.__player = player
		self.__root_window = player.window.get_screen().get_root_window()
		self.__icon_position = None
		self.__cursor_position = None
		self.__signal_id_1 = self.__signal_id_2 = None
		return

	def __monitor_cursor_movement(self):
		"""
		Monitor mouse pointer position.

		@param self: Reference to the Cursor instance.
		@type self: A Cursor object.

		@return: True to call this method repeatedly.
		@rtype: A Boolean object.
		"""
		from gobject import idle_add
		idle_add(self.__response)
		x, y, data = self.__root_window.get_pointer()
		from operator import eq, not_, contains
		if eq(self.__cursor_position, (x, y)): return True
		self.__cursor_position = x, y
		if not_(self.__icon_position): return True
		self.__status_window_visibility(x, y)
		return True

	def __status_window_visibility(self, x ,y):
		"""
		Determine whether or not to emit a signal to show status window.

		@param self: Reference to the Cursor instance.
		@type self: A Cursor object.

		@param x: xcordinate of the mouse pointer.
		@type x: An Integer object.

		@param y: ycordinate of the mouse pointer.
		@type y: An Integer object.
		"""
		from operator import contains
		xcord, ycord, width, height = self.__icon_position
		x_exp = contains(xrange(xcord, xcord+width+1), x)
		y_exp = contains(xrange(ycord, ycord+height+1), y)
		if x_exp and y_exp:
			self.__manager.emit("show-status-window", self.__icon_position)
		else:
			self.__manager.emit("hide-status-window")
		from gobject import idle_add, source_remove
		try:
			source_remove(self.__status_window_visibility_id)
		except:
			pass
		self.__status_window_visibility_id = idle_add(self.__response)
		return

	def __response(self):
		self.__player.response()
		return False

	def __precompile_methods(self):
		"""
		Use psyco to optimize methods.

		@param self: Reference to the Cursor instance.
		@type self: A Cursor object.
		"""
		try:
			from psyco import bind
			bind(self.__status_window_visibility)
			bind(self.__monitor_cursor_movement)
		except ImportError:
			pass
		return

	def __icon_position_cb(self, manager, data):
		"""
		Handles callback when the "icon-position" signal is emitted.

		@param self: Reference to the Cursor instance.
		@type self: A Cursor object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param data: Position of the StatusIcon and related information.
		@type data: A Tuple object.
		"""
		from operator import not_
		if not_(data): return
		self.__icon_position = data[1]
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Cursor instance.
		@type self: A Cursor object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__manager)
		disconnect_signal(self.__signal_id_2, self.__manager)
		delete_attributes(self)
		del self
		self = None
		return
